==> 2.0.3.tar.gz: nci2/nci2/neocities.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
############################################
#                        _   ___           #
#            _ _    __  (_) |_  )          #
#           | ' \  / _| | |  / /           #
#           |_||_| \__| |_| /___|          #
#                                          #
############################################
# nci2: blog generator for neocities. it's #
# is an improved version of the old nci    #
# web script by nokoru <nokoru@disroot.org>#
#                                          #
# license: unlicense                       #
############################################
import os
import getpass
from .neocities_requests import NeocitiesUser
from .format import *
from .files import *
from .config import *

# Prompt user for their credentials
def nci2_login(use_api_key):
    credentials = []
    cfg = read_cfg_file()
    credentials.append(cfg["username"])
    credentials.append("")
    auth_type = "Password" if not use_api_key else "API key"
    printf("{yellow}*{r} " + auth_type + " for {green}" + credentials[0] + ".neocities.org{r}:")
    while credentials[1].strip() == "":
        try:
            credentials[1] = getpass.getpass(">")
        except (KeyboardInterrupt, SystemExit):
            printf("")
            exit(0)
        if credentials[1].strip() == "":
            printf("{red}x{r} Invalid " + auth_type + ". Please try again:")
    return credentials

# Download website files from neocities
def nci2_pull(force_overwrite, use_api_key):
    printf("{yellow}*{r} The {blue}pull{r} command lets you synchronize/download your website files from neocities.")
    if force_overwrite:
        printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files in public_html/ will be overwritten!{r}")
    
    auth_type = "password" if not use_api_key else "API key"
    credentials = nci2_login(use_api_key)
    nc = NeocitiesUser(credentials[0], credentials[1] if not use_api_key else "", credentials[1] if use_api_key else "")
    printf("{yellow}*{r} Checking credentials...")
    if not nc.credentials_ok() or not nc.login_ok():
        printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        exit(1)
    printf(u"{green}[{b}✓{r}{green}]{r} Successfully logged in!")
    printf("{yellow}*{r} Retrieving file list...")
    files = nc.get_file_list()
    if type(files) is dict:
        printf("{yellow}*{r} Pulling...")
        downloaded=0
        for f in files["files"]:
            if not f["is_directory"]:
                printf("{yellow}*{r} Downloading \"" + f["path"] + "\"...")
            path = "public_html/" + f["path"]
            if os.path.isfile(path) and not force_overwrite:
                yn=""
                printf("{yellow}[{b}!{r}{yellow}]{r} The file " + f["path"] + " already exists. Do you want to overwrite it? (y/n)")
                while yn != "y" and yn != "n":
                    try:
                        yn = input(">").lower()
                    except (KeyboardInterrupt, SystemExit):
                        printf("")
                        exit(0)
                    if yn != "y" and yn != "n":
                        printf("{red}x{r} Please answer just {green}y{r} or {green}n{r}. Do you want to overwrite the file?")
                if yn == "n":
                    printf("{yellow}[{b}!{r}{yellow}]{r} File \"" + f["path"] +"\" skipped.")
                    continue
            
            # Download the file
            if f["is_directory"]:
                printf("{yellow}*{r} \"" + f["path"] + "\" is a directory. Creating it...")
                try:
                    mkdir(path)
                except BadMkdir:
                    printf("{yellow}[{b}!{r}{yellow}]{r} The directory already exists; nci2 skips its creation.")
            else:
                result = nc.download_file(f["path"], path)
                if result == 200 and (os.path.isfile(path) or os.path.isdir(path)):
                    printf(u"{green}[{b}✓{r}{green}]{r} File \"" + f["path"] + "\" successfully downloaded.")
                    downloaded+=1
                else:
                    error_text = ""
                    if result == 404:
                        error_text = "file not found"
                    elif result == 403:
                        error_text = "unauthorized access"
                    else:
                        error_text = "unknown error"
                    printf("{red}[{b}error{r}{red}]{r} Couldn't download the file \"" + f["path"] +"\" (" + error_text + ")")
        printf(u"{green}[{b}✓{r}{green}] {b}{blue}Pull{green} command completed (" + str(downloaded) + " files downloaded).")
        printf("OK.")
    else:
        if files == 403:
            printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        else:
            printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(files) + "{r}. I'm really sorry >.<")

# These two variables must be global and they're really important!
upload_list = []
delete_list = []
dirdel_list = []

def init_file_comparison(neocities_files):
    # Write "delete_list"
    for nc_file in neocities_files["files"]:
        """
        All files are listed for deletion until their local existence is verified, exceptions are the index.html file,
        the 404 (not_found.html) file, and hidden files.
        
        Directory filenames are added into a different list, so we delete their files one by one to avoid errors
        """
        if nc_file["is_directory"]:
            dirdel_list.append(nc_file["path"])
        
        if nc_file["path"] != "index.html" and nc_file["path"] != "not_found.html" and not nc_file["is_directory"] and nc_file["path"].find(".") != 0:
            delete_list.append(nc_file["path"])
    
    # Start recursive file comparison
    compare_files(neocities_files)

# Recursively compare neocities files with local files (public_html/*)
def compare_files(neocities_files, dirname="public_html/"):
    for f in os.listdir(dirname):
        # nci2 ignores local hidden files
        if f.find(".") == 0:
            continue
        
        # full path
        local_path = dirname + f
        neocities_path = local_path.replace("public_html/", "", 1)
        
        # If f is a directory, compare its files
        if os.path.isdir(local_path):
            if neocities_path in dirdel_list:
                dirdel_list.remove(neocities_path)
            compare_files(neocities_files, local_path + "/")
            continue
        
        # Get the SHA1 sum of the current file
        f_sha1_hash = sha1sum(local_path)
        # Is this file new? (default: yes)
        is_new = True
        
        # Check if the current file will be uploaded, deleted or just left as it is.
        for nc_file in neocities_files["files"]:
            if nc_file["path"] == neocities_path:
                is_new = False
                if nc_file["path"] in delete_list:
                    delete_list.remove(nc_file["path"])
                
                # Compare file hashes
                if nc_file["sha1_hash"] != f_sha1_hash:
                    upload_list.append(neocities_path)
        
        # If the file is new, then upload it
        if is_new:
            upload_list.append(neocities_path)

# Push files and recursively push directories in public_html/
def nci2_push(use_api_key):
    printf("{yellow}*{r} You are about to upload ({blue}push{r}) your entire website ({green}public_html/*{r}) to {b}neocities{r}!")
    
    auth_type = "password" if not use_api_key else "API key"
    credentials = nci2_login(use_api_key)
    nc = NeocitiesUser(credentials[0], credentials[1] if not use_api_key else "", credentials[1] if use_api_key else "")
    printf("{yellow}*{r} Checking credentials...")
    if not nc.credentials_ok() or not nc.login_ok():
        printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        exit(1)
    printf(u"{green}[{b}✓{r}{green}] {b}Successfully logged in!{r}")
    printf("{yellow}*{r} Retrieving file list...")
    # Get list of user's files from neocities
    neocities_files = nc.get_file_list()
    if type(neocities_files) is not dict:
        if neocities_files == 403:
            printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
            exit(1)
        else:
            printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(neocities_files) + "{r}. I'm really sorry >.<")
            exit(1)
    
    # Compare neocities files with local files
    printf("{yellow}*{r} Comparing files...")
    init_file_comparison(neocities_files)
    upload_count = len(upload_list)
    delete_count = len(delete_list) + len(dirdel_list)
    
    # Wait for user confirmation
    printf("{yellow}***********************************************************{r}")
    printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files on neocities will be overwritten!{r}")
    printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files on neocities that don't exist in your public_html/ directory will be deleted!{r}")
    printf("{yellow}*{r} {green}" + str(upload_count) + "{r} " + ("files" if upload_count != 1 else "file") + " will be {green}uploaded{r} to neocities.")
    printf("{yellow}*{r} {red}" + str(delete_count) + "{r} " + ("files" if delete_count != 1 else "file") + " will be {red}deleted{r} from neocities.")
    yn=""
    printf("{yellow}*{r} Please tell me: Are you sure you want to continue? (y/n)")
    while yn != "y" and yn != "n":
        try:
            yn = input(">").lower()
        except (KeyboardInterrupt, SystemExit):
            printf("")
            exit(0)
        if yn != "y" and yn != "n":
            printf("{red}x{r} Please answer just {green}y{r} or {green}n{r}. Do you want to continue?")
    if yn == "n":
        printf("{yellow}*{r} Operation cancelled.")
        printf("OK.")
        exit(0)
    
    uploaded = 0
    not_uploaded = 0
    deleted = 0
    not_deleted = 0
    
    printf("{yellow}*{r} Uploading files...")
    for f in upload_list:
        printf("{yellow}*{r} Uploading \"" + f + "\"...")
        upload_result = nc.upload_file("public_html/" + f, f)
        if type(upload_result) is not dict:
            if upload_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(upload_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if upload_result["result"] == "success":
                printf("{green}[{b}+{r}{green}]{r} \"" + f +"\" successfully uploaded.")
                uploaded+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't upload the file: " + upload_result["message"])
                not_uploaded+=1
                
    printf("{yellow}*{r} Deleting files...")
    for f in delete_list:
        printf("{yellow}*{r} Deleting \"" + f + "\"...")
        delete_result = nc.delete_file(f)
        if type(delete_result) is not dict:
            if delete_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(delete_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if delete_result["result"] == "success":
                printf("{green}[{b}-{r}{green}]{r} \"" + f +"\" successfully deleted.")
                deleted+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't delete the file: " + delete_result["message"])
                not_deleted+=1
                
    # Delete empty directories
    for d in dirdel_list:
        printf("{yellow}*{r} Deleting \"" + d + "\"...")
        delete_result = nc.delete_file(d)
        if type(delete_result) is not dict:
            if delete_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(delete_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if delete_result["result"] == "success":
                printf("{green}[{b}-{r}{green}]{r} \"" + d +"\" successfully deleted.")
                deleted+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't delete the file: " + delete_result["message"])
                not_deleted+=1
    
    printf("{yellow}***********************************************************{r}")
    printf("{yellow}*{r} " + str(uploaded) + " files uploaded, " + str(not_uploaded) + " not uploaded.")
    printf("{yellow}*{r} " + str(deleted) + " files deleted, " + str(not_deleted) + " not deleted.")
    printf(u"{green}[{b}✓{r}{green}] {b}{blue}Push{green} command finished!{r}")
    printf("OK.")

==> 2.0.5.tar.gz: nci2/nci2/neocities.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
############################################
#                        _   ___           #
#            _ _    __  (_) |_  )          #
#           | ' \  / _| | |  / /           #
#           |_||_| \__| |_| /___|          #
#                                          #
############################################
# nci2: blog generator for neocities. it's #
# is an improved version of the old nci    #
# web script by nokoru <nokoru@disroot.org>#
#                                          #
# license: unlicense                       #
############################################
import os
import getpass
from .neocities_requests import NeocitiesUser
from .format import *
from .files import *
from .config import *

# Prompt user for their credentials
def nci2_login(use_api_key):
    credentials = []
    cfg = read_cfg_file()
    credentials.append(cfg["username"])
    credentials.append("")
    auth_type = "Password" if not use_api_key else "API key"
    printf("{yellow}*{r} " + auth_type + " for {green}" + credentials[0] + ".neocities.org{r}:")
    while credentials[1].strip() == "":
        try:
            credentials[1] = getpass.getpass(">")
        except (KeyboardInterrupt, SystemExit):
            printf("")
            exit(0)
        if credentials[1].strip() == "":
            printf("{red}x{r} Invalid " + auth_type + ". Please try again:")
    return credentials

# Download website files from neocities
def nci2_pull(force_overwrite, use_api_key):
    printf("{yellow}*{r} The {blue}pull{r} command lets you synchronize/download your website files from neocities.")
    if force_overwrite:
        printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files in public_html/ will be overwritten!{r}")
    
    auth_type = "password" if not use_api_key else "API key"
    credentials = nci2_login(use_api_key)
    nc = NeocitiesUser(credentials[0], credentials[1] if not use_api_key else "", credentials[1] if use_api_key else "")
    printf("{yellow}*{r} Checking credentials...")
    if not nc.credentials_ok() or not nc.login_ok():
        printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        exit(1)
    printf(u"{green}[{b}✓{r}{green}]{r} Successfully logged in!")
    printf("{yellow}*{r} Retrieving file list...")
    files = nc.get_file_list()
    if type(files) is dict:
        printf("{yellow}*{r} Pulling...")
        downloaded=0
        for f in files["files"]:
            if not f["is_directory"]:
                printf("{yellow}*{r} Downloading \"" + f["path"] + "\"...")
            path = "public_html/" + f["path"]
            if os.path.isfile(path) and not force_overwrite:
                yn=""
                printf("{yellow}[{b}!{r}{yellow}]{r} The file " + f["path"] + " already exists. Do you want to overwrite it? (y/n)")
                while yn != "y" and yn != "n":
                    try:
                        yn = input(">").lower()
                    except (KeyboardInterrupt, SystemExit):
                        printf("")
                        exit(0)
                    if yn != "y" and yn != "n":
                        printf("{red}x{r} Please answer just {green}y{r} or {green}n{r}. Do you want to overwrite the file?")
                if yn == "n":
                    printf("{yellow}[{b}!{r}{yellow}]{r} File \"" + f["path"] +"\" skipped.")
                    continue
            
            # Download the file
            if f["is_directory"]:
                printf("{yellow}*{r} \"" + f["path"] + "\" is a directory. Creating it...")
                try:
                    mkdir(path)
                except BadMkdir:
                    printf("{yellow}[{b}!{r}{yellow}]{r} The directory already exists; nci2 skips its creation.")
            else:
                result = nc.download_file(f["path"], path)
                if result == 200 and (os.path.isfile(path) or os.path.isdir(path)):
                    printf(u"{green}[{b}✓{r}{green}]{r} File \"" + f["path"] + "\" successfully downloaded.")
                    downloaded+=1
                else:
                    error_text = ""
                    if result == 404:
                        error_text = "file not found"
                    elif result == 403:
                        error_text = "unauthorized access"
                    else:
                        error_text = "unknown error"
                    printf("{red}[{b}error{r}{red}]{r} Couldn't download the file \"" + f["path"] +"\" (" + error_text + ")")
        printf(u"{green}[{b}✓{r}{green}] {b}{blue}Pull{green} command completed (" + str(downloaded) + " files downloaded).")
        printf("OK.")
    else:
        if files == 403:
            printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        else:
            printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(files) + "{r}. I'm really sorry >.<")

# These three variables must be global and they're really important!
upload_list = []
delete_list = []
dirdel_list = []

def init_file_comparison(neocities_files):
    # Write "delete_list"
    for nc_file in neocities_files["files"]:
        """
        All files are listed for deletion until their local existence is verified, exceptions are the index.html file,
        the 404 (not_found.html) file, and hidden files.
        
        Directory filenames are added into a different list, so we delete their files one by one to avoid errors
        """
        if nc_file["is_directory"]:
            dirdel_list.append(nc_file["path"])
        
        if nc_file["path"] != "index.html" and nc_file["path"] != "not_found.html" and not nc_file["is_directory"] and nc_file["path"].find(".") != 0:
            delete_list.append(nc_file["path"])
    
    # Start recursive file comparison
    compare_files(neocities_files)

# Recursively compare neocities files with local files (public_html/*)
def compare_files(neocities_files, dirname="public_html/"):
    for f in os.listdir(dirname):
        # nci2 ignores local hidden files
        if f.find(".") == 0:
            continue
        
        # full path
        local_path = dirname + f
        neocities_path = local_path.replace("public_html/", "", 1)
        
        # If f is a directory, compare its files
        if os.path.isdir(local_path):
            if neocities_path in dirdel_list:
                dirdel_list.remove(neocities_path)
            compare_files(neocities_files, local_path + "/")
            continue
        
        # Get the SHA1 sum of the current file
        f_sha1_hash = sha1sum(local_path)
        # Is this file new? (default: yes)
        is_new = True
        
        # Check if the current file will be uploaded, deleted or just left as it is.
        for nc_file in neocities_files["files"]:
            if nc_file["path"] == neocities_path:
                is_new = False
                if nc_file["path"] in delete_list:
                    delete_list.remove(nc_file["path"])
                
                # Compare file hashes
                if nc_file["sha1_hash"] != f_sha1_hash:
                    upload_list.append(neocities_path)
        
        # If the file is new, then upload it
        if is_new:
            upload_list.append(neocities_path)

# Push files and recursively push directories in public_html/
def nci2_push(use_api_key):
    printf("{yellow}*{r} You are about to upload ({blue}push{r}) your entire website ({green}public_html/*{r}) to {b}neocities{r}!")
    
    auth_type = "password" if not use_api_key else "API key"
    credentials = nci2_login(use_api_key)
    nc = NeocitiesUser(credentials[0], credentials[1] if not use_api_key else "", credentials[1] if use_api_key else "")
    printf("{yellow}*{r} Checking credentials...")
    if not nc.credentials_ok() or not nc.login_ok():
        printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        exit(1)
    printf(u"{green}[{b}✓{r}{green}] {b}Successfully logged in!{r}")
    printf("{yellow}*{r} Retrieving file list...")
    # Get list of user's files from neocities
    neocities_files = nc.get_file_list()
    if type(neocities_files) is not dict:
        if neocities_files == 403:
            printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
            exit(1)
        else:
            printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(neocities_files) + "{r}. I'm really sorry >.<")
            exit(1)
    
    # Compare neocities files with local files
    printf("{yellow}*{r} Comparing files...")
    init_file_comparison(neocities_files)
    upload_count = len(upload_list)
    delete_count = len(delete_list) + len(dirdel_list)
    
    # Wait for user confirmation
    printf("{yellow}***********************************************************{r}")
    printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files on neocities will be overwritten!{r}")
    printf("{yellow}[{b}!{r}{yellow}]{r} {red}{b}Existing files on neocities that don't exist in your public_html/ directory will be deleted!{r}")
    printf("{yellow}*{r} {green}" + str(upload_count) + "{r} " + ("files" if upload_count != 1 else "file") + " will be {green}uploaded{r} to neocities.")
    printf("{yellow}*{r} {red}" + str(delete_count) + "{r} " + ("files" if delete_count != 1 else "file") + " will be {red}deleted{r} from neocities.")
    yn=""
    printf("{yellow}*{r} Please tell me: Are you sure you want to continue? (y/n)")
    while yn != "y" and yn != "n":
        try:
            yn = input(">").lower()
        except (KeyboardInterrupt, SystemExit):
            printf("")
            exit(0)
        if yn != "y" and yn != "n":
            printf("{red}x{r} Please answer just {green}y{r} or {green}n{r}. Do you want to continue?")
    if yn == "n":
        printf("{yellow}*{r} Operation cancelled.")
        printf("OK.")
        exit(0)
    
    uploaded = 0
    not_uploaded = 0
    deleted = 0
    not_deleted = 0
    
    printf("{yellow}*{r} Uploading files...")
    for f in upload_list:
        printf("{yellow}*{r} Uploading \"" + f + "\"...")
        upload_result = nc.upload_file("public_html/" + f, f)
        if type(upload_result) is not dict:
            if upload_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(upload_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if upload_result["result"] == "success":
                printf("{green}[{b}+{r}{green}]{r} \"" + f +"\" successfully uploaded.")
                uploaded+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't upload the file: " + upload_result["message"])
                not_uploaded+=1

    printf("{yellow}*{r} Deleting files...")
    for f in delete_list:
        printf("{yellow}*{r} Deleting \"" + f + "\"...")
        delete_result = nc.delete_file(f)
        if type(delete_result) is not dict:
            if delete_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(delete_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if delete_result["result"] == "success":
                printf("{green}[{b}-{r}{green}]{r} \"" + f +"\" successfully deleted.")
                deleted+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't delete the file: " + delete_result["message"])
                not_deleted+=1
                
    # Delete empty directories
    for d in dirdel_list:
        printf("{yellow}*{r} Deleting \"" + d + "\"...")
        delete_result = nc.delete_file(d)
        if type(delete_result) is not dict:
            if delete_result == 403:
                printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
                exit(1)
            else:
                printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(delete_result) + "{r}. I'm really sorry >.<")
                exit(1)
        else:
            if delete_result["result"] == "success":
                printf("{green}[{b}-{r}{green}]{r} \"" + d +"\" successfully deleted.")
                deleted+=1
            else:
                printf("{red}[{b}x{r}{red}]{r} Couldn't delete the file: " + delete_result["message"])
                not_deleted+=1
    
    printf("{yellow}***********************************************************{r}")
    printf("{yellow}*{r} " + str(uploaded) + " files uploaded, " + str(not_uploaded) + " not uploaded.")
    printf("{yellow}*{r} " + str(deleted) + " files deleted, " + str(not_deleted) + " not deleted.")
    printf(u"{green}[{b}✓{r}{green}] {b}{blue}Push{green} command finished!{r}")
    printf("OK.")

# These three variables must be global!
local_new_list = []
local_mod_list = []
local_del_list = []
hashes_dict = {}

# the functions below will be exclusively used for the status command!
def init_file_comparison_status(neocities_files, show_hashes):
    # Write del_list
    for nc_file in neocities_files["files"]:
        """
        All files are listed for deletion until their local existence is verified, exceptions are the index.html file,
        the 404 (not_found.html) file, and hidden files.
        """
        if nc_file["is_directory"]:
            continue
        
        if nc_file["path"] != "index.html" and nc_file["path"] != "not_found.html" and nc_file["path"].find(".") != 0:
            local_del_list.append(nc_file["path"])
    
    # Start recursive file comparison
    compare_files_status(neocities_files, show_hashes)

# Recursively compare neocities files with local files (public_html/*)
def compare_files_status(neocities_files, show_hashes, dirname = "public_html/"):
    for f in os.listdir(dirname):
        # nci2 ignores local hidden files
        if f.find(".") == 0:
            continue
        
        # full path
        local_path = dirname + f
        neocities_path = local_path.replace("public_html/", "", 1)
        
        # If f is a directory, compare its files
        if os.path.isdir(local_path):
            compare_files_status(neocities_files, show_hashes, local_path + "/")
            continue
        
        # Get the SHA1 sum of the current file
        f_sha1_hash = sha1sum(local_path)
        # Is this file new? (default: yes)
        is_new = True
        
        # Check if the current file will be uploaded, deleted or just left as it is.
        for nc_file in neocities_files["files"]:
            if nc_file["path"] == neocities_path:
                is_new = False
                if nc_file["path"] in local_del_list:
                    local_del_list.remove(nc_file["path"])
                
                # Compare file hashes
                if nc_file["sha1_hash"] != f_sha1_hash:
                    local_mod_list.append(neocities_path)
                    if show_hashes:
                        hashes_dict[neocities_path] = (f_sha1_hash, nc_file["sha1_hash"]) # dict["file.txt"] = (local_hash, neocities_hash)
        
        # If the file is new, then upload it
        if is_new:
            local_new_list.append(neocities_path)

def nci2_status(display_type, show_hashes, use_api_key):
    auth_type = "password" if not use_api_key else "API key"
    credentials = nci2_login(use_api_key)
    nc = NeocitiesUser(credentials[0], credentials[1] if not use_api_key else "", credentials[1] if use_api_key else "")
    printf("{yellow}*{r} Checking credentials...")
    if not nc.credentials_ok() or not nc.login_ok():
        printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
        exit(1)
    printf(u"{green}[{b}✓{r}{green}] {b}Successfully logged in!{r}")
    printf("{yellow}*{r} Retrieving file list...")
    neocities_files = nc.get_file_list()
    if type(neocities_files) is not dict:
        if neocities_files == 403:
            printf("{red}[{b}error{r}{red}]{r} Invalid username or " + auth_type + ". Please check your credentials and try again.")
            exit(1)
        else:
            printf("{red}[{b}error{r}{red}]{r} Oops! An unexpected error occurred. The HTTP error code is {red}{b}" + str(neocities_files) + "{r}. I'm really sorry >.<")
            exit(1)
    # Compare neocities files with local files
    printf("{yellow}*{r} Comparing files...")
    init_file_comparison_status(neocities_files, show_hashes)

    new_count = len(local_new_list)
    mod_count = len(local_mod_list)
    del_count = len(local_del_list)

    new_count_str = ("{green}" + str(new_count) + "{r} new") if display_type == "all" or display_type == "new" else ""
    mod_count_str = ("{blue}" + str(mod_count) + "{r} modified") if display_type == "all" or display_type == "mod" else ""
    del_count_str = ("{red}" + str(del_count) + "{r} deleted") if display_type == "all" or display_type == "del" else ""

    # print results
    printf("{yellow}***********************************************************{r}")
    printf("{yellow}[{b}!{r}{yellow}]{r} Files successfully compared. Detected changes:")
    if new_count == 0 and mod_count == 0 and del_count == 0:
        printf(u"\n{green}[{b}✓{r}{green}] No changes. Everything's clean and up-to-date.{r}")
    else:
        if display_type == "all" or display_type == "new":
            printf("\n{green}[{b}+{r}{green}]{r} New files ({green}" + str(new_count) + "{r}):")
            for f in local_new_list:
                printf("\t{green}" + f + "{r}")
        if display_type == "all" or display_type == "mod":
            printf("\n{blue}[{b}*{r}{blue}]{r} Modified files ({blue}" + str(mod_count) + "{r}):")
            for f in local_mod_list:
                printf("\t{blue}" + f + "{r}")
                if show_hashes:
                    printf("\t\t{blue}(SHA1 local: " + hashes_dict[f][0] + ", remote: " + hashes_dict[f][1] + "){r}")
        if display_type == "all" or display_type == "del":
            printf("\n{red}[{b}-{r}{red}]{r} Deleted/Moved files ({red}" + str(del_count) + "{r}):")
            for f in local_del_list:
                printf("\t{red}" + f + "{r}")
        printf("\n{yellow}*{r} " + new_count_str + (", " if mod_count_str != "" and new_count_str != "" else "") + mod_count_str + (", " if del_count_str != "" and mod_count_str != "" else "") +  del_count_str + ".")
    printf("{yellow}***********************************************************{r}")
    printf("OK.")
